/** 
* Copyright 2011 The Apache Software Foundation
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
* 
* @author Felipe Oliveira (http://mashup.fm)
* 
*/
package models;

import javax.persistence.Entity;

import play.data.validation.Required;
import play.db.jpa.Model;
import controllers.CRUD.Hidden;

/**
 * The Class OAuthServiceProvider.
 */
@Entity
public class OAuthServiceProvider extends Model {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	@Required
	public String name;

	/** The request token url. */
	@Required
	public String requestTokenURL;

	/** The user authorization url. */
	@Required
	public String userAuthorizationURL;

	/** The access token url. */
	@Required
	public String accessTokenURL;

	/** The base url. */
	@Hidden
	public String baseURL;

	/**
	 * Instantiates a new o auth service provider.
	 *
	 * @param requestTokenURL the request token url
	 * @param userAuthorizationURL the user authorization url
	 * @param accessTokenURL the access token url
	 */
	public OAuthServiceProvider(String requestTokenURL,
			String userAuthorizationURL, String accessTokenURL) {
		this.requestTokenURL = requestTokenURL;
		this.userAuthorizationURL = userAuthorizationURL;
		this.accessTokenURL = accessTokenURL;
	}

	/* (non-Javadoc)
	 * @see play.db.jpa.JPABase#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((accessTokenURL == null) ? 0 : accessTokenURL.hashCode());
		result = prime * result
				+ ((requestTokenURL == null) ? 0 : requestTokenURL.hashCode());
		result = prime
				* result
				+ ((userAuthorizationURL == null) ? 0 : userAuthorizationURL
						.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see play.db.jpa.JPABase#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		OAuthServiceProvider other = (OAuthServiceProvider) obj;
		if (accessTokenURL == null) {
			if (other.accessTokenURL != null) {
				return false;
			}
		} else if (!accessTokenURL.equals(other.accessTokenURL)) {
			return false;
		}
		if (requestTokenURL == null) {
			if (other.requestTokenURL != null) {
				return false;
			}
		} else if (!requestTokenURL.equals(other.requestTokenURL)) {
			return false;
		}
		if (userAuthorizationURL == null) {
			if (other.userAuthorizationURL != null) {
				return false;
			}
		} else if (!userAuthorizationURL.equals(other.userAuthorizationURL)) {
			return false;
		}
		return true;
	}
	
	public String toString() {
		return name;
	}

}
